import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pytessng",
    version="0.3.11",
    author="yang",
    author_email="17315487709@163.com",
    description="tessng with python3.6",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    install_requires=[
        "attrs==22.2.0",
        "certifi==2021.5.30",
        "charset-normalizer==2.0.12",
        "click==8.0.4",
        "click-plugins==1.1.1",
        "cligj==0.7.2",
        "colorama==0.4.5",
        "commonroad-io==2020.2",
        "cycler==0.11.0",
        "dbfread==2.0.7",
        "decorator==4.4.2",
        "Fiona==1.8.22",
        "geopandas==0.9.0",
        "idna==3.6",
        "importlib-metadata==4.8.3",
        "importlib-resources==5.4.0",
        "kafka-python==2.0.2",
        "kiwisolver==1.3.1",
        "lxml==4.9.4",
        "matplotlib==3.3.4",
        "MouseInfo==0.1.3",
        "munch==4.0.0",
        "networkx==2.5.1",
        "numpy==1.19.5",
        "osmnx==1.1.1",
        "pandas==1.1.5",
        "Pillow==8.4.0",
        "PyAutoGUI==0.9.54",
        "PyGetWindow==0.0.9",
        "PyMsgBox==1.0.9",
        "pyparsing==3.0.7",
        "pyperclip==1.8.2",
        "pyproj==3.0.1",
        "PyRect==0.2.0",
        "PyScreeze==0.1.30",
        "pyshp==2.3.1",
        "PySide2==5.15.2.1",
        "python-dateutil==2.8.2",
        "pytweening==1.0.7",
        "pytz==2023.3.post1",
        "requests==2.27.1",
        "Rtree==0.9.7",
        "scipy==1.5.4",
        "Shapely==1.8.5.post1",
        "shiboken2==5.15.2.1",
        "six==1.16.0",
        "tqdm==4.64.1",
        "typing_extensions==4.1.1",
        "urllib3==1.26.18",
        "wincertstore==0.2",
        "xlrd==1.2.0",
        "zipp==3.6.0",
    ],
    python_requires='>=3.6, <3.7',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    zip_safe=False,
    include_package_data=True,
)
